// ignore_for_file: use_key_in_widget_constructors

import 'dart:async';
import 'dart:developer';
import 'dart:io';

import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:network_info_plus/network_info_plus.dart';
import 'screens/bc_device_screen.dart';
import 'screens/bp_device_screen.dart';
import 'screens/bt_device_screen.dart';
import 'screens/log_screen.dart';
import 'screens/pr_device_screen.dart';
import 'screens/oc_device_screen.dart';
import 'screens/ms_device_screen.dart';
import 'screens/zi_device_screen.dart';
import 'screens/zl_device_screen.dart';
import 'screens/be_device_screen.dart';
import 'screens/bg_device_screen.dart';
import 'screens/dd_device_screen.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return GetMaterialApp(
      title: 'Flutter Demo',
      debugShowCheckedModeBanner: false,
      theme: ThemeData(
        // This is the theme of your application.
        //
        // Try running your application with "flutter run". You'll see the
        // application has a blue toolbar. Then, without quitting the app, try
        // changing the primarySwatch below to Colors.green and then invoke
        // "hot reload" (press "r" in the console where you ran "flutter run",
        // or simply save your changes to "hot reload" in a Flutter IDE).
        // Notice that the counter didn't reset back to zero; the application
        // is not restarted.
        primarySwatch: Colors.blue,
      ),
      home: MyHomePage(),
    );
  }
}

class MyHomePage extends StatefulWidget {
  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  PlatformDevices platformDevices = PlatformDevices.empty();
  TextEditingController ipC = TextEditingController(text: "192.168.9.101");
  TextEditingController portC = TextEditingController(text: "35000");
  bool platformConnect = false;

  bool loading = false;
  bool connecting = false;
  bool initing = false;
  bool initingAll = false;


  goToDeviceDetails(dynamic device) {
    switch (device.runtimeType) {
      case BpDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BpDeviceScreen(device: device)));
        break;
      case BtDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BtDeviceScreen(device: device)));
        break;
      case BcDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BcDeviceScreen(device: device)));
        break;
      case BeDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BeDeviceScreen(device: device)));
        break;
      case OcDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => OcDeviceScreen(device: device)));
        break;
      case MsDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => MsDeviceScreen(device: device)));
        break;
      case PrDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => PrDeviceScreen(device: device)));
        break;
      case BgDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BgDeviceScreen(device: device)));
        break;
      case DdDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => DdDeviceScreen(device: device)));
        break;
      case ZlDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => ZlDeviceScreen(device: device)));
        break;
      case ZiDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => ZiDeviceScreen(device: device)));
        break;
    }
  }

  @override
  Widget build(BuildContext context) {
    ThemeData theme = Theme.of(context);

    return Scaffold(
      appBar: AppBar(
        title: const Text("CUPPS"),
        actions: [
          IconButton(
              onPressed: () async {
                Navigator.push(context, MaterialPageRoute(builder: (c) => const LogScreen()));

                // XMLUtil.tryParseXML();
                // setState(() {
                //   loading = true;
                // });
                // await XMLUtil.waitWhile(() => XMLUtil().asyncDone).then((value) {
                //   print("here");
                //   setState(() {
                //     loading = false;
                //   });
                // });
              },
              icon: loading ? const Icon(Icons.send) : const Icon(Icons.bug_report_outlined))
        ],
      ),
      body: Column(
        children: [
          const Divider(),
          Container(
            padding: const EdgeInsets.symmetric(horizontal: 24),
            width: MediaQuery.of(context).size.width,
            color: Colors.white,
            child: Column(
              children: [
                platformConnect
                    ? Row(
                  children: [
                    Expanded(child: Text("Connected to ${ipC.text}:${portC.text}")),
                    TextButton(
                        onPressed: () async {
                          // setState(() {
                          //   platformConnect = false;
                          //   platformDevices = PlatformDevices.empty();
                          // });
                          CUPPS.disconnectFromPlatform();
                        },
                        child: const Text("Disconnect")),
                  ],
                )
                    : Row(
                  children: [
                    Expanded(
                        flex: 4,
                        child: CupertinoTextField(
                          keyboardType: const TextInputType.numberWithOptions(signed: true, decimal: true),
                          controller: ipC,
                          placeholder: "ip e.g. 192.168.1.7",
                        )),
                    const SizedBox(width: 12),
                    Expanded(
                        flex: 2,
                        child: CupertinoTextField(
                          keyboardType: const TextInputType.numberWithOptions(signed: true),
                          controller: portC,
                          placeholder: "port e.g. 7535",
                        )),
                    const SizedBox(width: 12),
                    TextButton(
                        onPressed: () async {
                          setState(() {
                            connecting = true;
                          });
                          CUPPS
                              .connectToPlatform(
                              ip: ipC.text,
                              port: int.tryParse(portC.text) ?? 0,
                              airlineCode: "TK",
                              notifier: () {
                                setState(() {
                                  platformDevices = CUPPS().platformDevices;
                                });
                              },
                              onPlatformConnectionChange: (bool v) {
                                setState(() {
                                  platformConnect = v;
                                });
                              })
                              .then((value) {
                            setState(() {
                              connecting = false;
                            });
                            if (value) {
                              setState(() {
                                initing =true;
                              });
                              CUPPS.authenticate().then((value){
                                setState(() {
                                  initing =false;
                                });
                                if(value){
                                  setState(() {
                                    initingAll =true;
                                  });
                                  CUPPS.connectToAvailableNeededDevices().then((value){
                                    setState(() {
                                      initingAll =false;
                                    });
                                  });
                                }
                              });
                              // var req = InterfaceLevelsAvailableRequest(
                              //     hsXsdVersion: '01.03', messageID: CUPPS().currentMessageID);
                              // CUPPS.send(xml: req.toXML());
                            }
                          });
                        },
                        child: connecting ? const Text("...") : const Text("Connect")),
                  ],
                ),
              ],
            ),
          ),
          const Divider(),
          Text("$initing"),
          Text("$initingAll"),
          const SizedBox(width: 12),
          // TextButton(
          //     onPressed: () async {
          //       SocketService.initialize(ip: "192.168.1.7", port: 7535);
          //     },
          //     child: connecting ? const Text("...") : const Text("Connect")),
          // TextButton(
          //     onPressed: () async {
          //       InterfaceLevelsAvailableRequest req = InterfaceLevelsAvailableRequest(messageID: 1, hsXsdVersion: "01.03");
          //
          //       final result = await SocketService.sendSocketMessage(req.toXML(), 1).timeout(Duration(seconds: 4), onTimeout: () {
          //         print("timeout");
          //         return "timeOut";
          //       });
          //       print(result);
          //     },
          //     child: const Text("Send")),
          const Divider(),
          Expanded(
              child: GridView(
                padding: const EdgeInsets.symmetric(horizontal: 12),
                gridDelegate:
                const SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 3, crossAxisSpacing: 12, mainAxisSpacing: 12),
                children: platformDevices.bpDevices
                    .map((e) => DeviceWidget(
                  device: e,
                  onSelect: () async {
                    goToDeviceDetails(e);
                  },
                ))
                    .toList() +
                    platformDevices.btDevices
                        .map((e) => DeviceWidget(
                      device: e,
                      onSelect: () {
                        goToDeviceDetails(e);
                      },
                    ))
                        .toList() +
                    platformDevices.bcDevices
                        .map((e) => DeviceWidget(
                      device: e,
                      onSelect: () {
                        goToDeviceDetails(e);
                      },
                    ))
                        .toList() +
                    platformDevices.bgDevices
                        .map((e) => DeviceWidget(
                      device: e,
                      onSelect: () {
                        goToDeviceDetails(e);
                      },
                    ))
                        .toList() +
                    // platformDevices.beDevices
                    //     .map((e) => DeviceWidget(
                    //   device: e,
                    //   onSelect: () {
                    //     goToDeviceDetails(e);
                    //   },
                    // ))
                    //     .toList() +
                    // platformDevices.ddDevices
                    //     .map((e) => DeviceWidget(
                    //   device: e,
                    //   onSelect: () {
                    //     goToDeviceDetails(e);
                    //   },
                    // ))
                    //     .toList() +
                    platformDevices.msDevices
                        .map((e) => DeviceWidget(
                      device: e,
                      onSelect: () {
                        goToDeviceDetails(e);
                      },
                    ))
                        .toList() +
                    platformDevices.prDevices
                        .map((e) => DeviceWidget(
                      device: e,
                      onSelect: () {
                        goToDeviceDetails(e);
                      },
                    ))
                        .toList() +
                    // platformDevices.ziDevices
                    //     .map((e) => DeviceWidget(
                    //   device: e,
                    //   onSelect: () {
                    //     goToDeviceDetails(e);
                    //   },
                    // ))
                    //     .toList() +
                    // platformDevices.zlDevices
                    //     .map((e) => DeviceWidget(
                    //   device: e,
                    //   onSelect: () {
                    //     goToDeviceDetails(e);
                    //   },
                    // ))
                    //     .toList() +
                    platformDevices.ocDevices
                        .map((e) => DeviceWidget(
                      device: e,
                      onSelect: () {
                        goToDeviceDetails(e);
                      },
                    ))
                        .toList(),
              )),
          FutureBuilder(
              future: NetworkInfo().getWifiIP(),
              builder: (c, snapshot) {
                if (snapshot.connectionState == ConnectionState.done || snapshot.hasData) {
                  var data = snapshot.data;
                  return Text("Device local IP $data");
                }
                return const Text("Loading ip");
              }),
        ],
      ),
    );
  }
}

class DeviceWidget extends StatelessWidget {
  final Device device;
  final Function onSelect;

  const DeviceWidget({required this.device, required this.onSelect});

  @override
  Widget build(BuildContext context) {
    ThemeData theme = Theme.of(context);
    return TextButton(
        onPressed: () {
          onSelect();
        },
        child: Image.asset(device.deviceParameter.status.image, package: 'artemis_cupps'));
    // return
    //   Column(
    //   children: [
    //     ListTile(
    //       tileColor: device.deviceParameter.status.color,
    //       leading: Text("${device.deviceIndex} ${device.getType.name}"),
    //       trailing: Icon(Icons.circle, color: device.isConnected ? Colors.green : Colors.red),
    //       title: Text(device.deviceName),
    //       subtitle: Text("${device.deviceParameter.ipAndPort} ${device.deviceParameter.status}"),
    //     ),
    //     Row(
    //       mainAxisAlignment: MainAxisAlignment.spaceAround,
    //       children: [
    //         TextButton(
    //             onPressed: () {
    //               onConnect();
    //             },
    //             child: const Text("Connect")),
    //         TextButton(
    //             onPressed: () {
    //               initialize();
    //             },
    //             child: const Text("Initialize")),
    //         TextButton(onPressed: () {
    //           AeaRequestRequest req = AeaRequestRequest(text: DataExample.printAeaData);
    //           send(req.toXML());
    //         }, child: const Text("Send AEA")),
    //       ],
    //     )
    //   ],
    // );
  }
}
