import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:artemis_cupps/artemis_cupps.dart';

class BgDeviceScreen extends StatefulWidget {
  final BgDevice device;

  const BgDeviceScreen({required this.device, Key? key}) : super(key: key);

  @override
  _BgDeviceScreenState createState() => _BgDeviceScreenState();
}

class _BgDeviceScreenState extends State<BgDeviceScreen> {
  List<String> receivedText = [];
  TextEditingController messageC = TextEditingController();

  @override
  void initState() {
    CUPPS.setBgListener((result) async{
      setState(() {
        receivedText.add(result.value);
      });
      await Future.delayed(const Duration(milliseconds: 200));
      return BgResponse(status: true,message: "OK Passenger");
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    BgDevice device = widget.device;
    return Scaffold(
      appBar: AppBar(
        title: Text("${device.deviceIndex} ${device.deviceName}"),
        actions: [IconButton(onPressed: (){
          device.refreshStatus();
        }, icon:const Icon(Icons.refresh))],
      ),
      body: Container(
        padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 12),
        child: Column(
          children: [
            Row(
              children: [
                Expanded(flex: 1, child: Image.asset(device.deviceParameter.status.image)),
                const SizedBox(width: 12),
                Expanded(
                    flex: 2,
                    child: Column(
                      children: [
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            const Text("Status"),
                            Text(
                              "${device.deviceParameter.status}",
                              style: const TextStyle(fontWeight: FontWeight.bold),
                            ),
                          ],
                        ),
                        const Divider(),
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            const Text("Address"),
                            Text(
                              "${device.deviceParameter.ipAndPort}",
                              style: const TextStyle(fontWeight: FontWeight.bold),
                            ),
                          ],
                        ),
                        const Divider(),
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [const Text("TCP Connection"), Icon(Icons.circle, color: device.isConnected ? Colors.green : Colors.red, size: 14)],
                        ),
                      ],
                    ))
              ],
            ),
            Row(
              children: [
                const Text("Available Modes"),
                const SizedBox(width: 12),
                Expanded(
                  child: Wrap(
                    direction: Axis.horizontal,
                    children: device.supportedInterfaceModes.map((e) => OutlinedButton(onPressed: () {}, child: Text(e.mode))).toList(),
                  ),
                ),
              ],
            ),
            const Divider(),
            Row(children: [
              Expanded(child: CupertinoTextField(
                controller: messageC,
                placeholder: "Message",
              )),
              const SizedBox(width: 24),
              ElevatedButton(
                onPressed: () {
                  device.displayText(messageC.text);
                },
                child: const Text("Show Message"),
              ),
            ],),
            ElevatedButton(
              onPressed: () {
                CUPPS.setBgListener((result) async{
                  setState(() {
                    receivedText.add(result.value);
                  });
                  await Future.delayed(const Duration(milliseconds: 200));
                  return BgResponse(status: true,message: "Accept Passenger");
                });
              },
              child: const Text("Accept Data"),
            ),
            ElevatedButton(
              onPressed: () {
                CUPPS.setBgListener((result) async{
                  setState(() {
                    receivedText.add(result.value);
                  });
                  await Future.delayed(const Duration(milliseconds: 200));
                  return BgResponse(status: false,message: "Reject Passenger");
                });
              },
              child: const Text("Reject Data"),
            ),

          ],
        ),
      ),
    );
  }
}
