import 'package:xml/xml.dart';

import 'classes.dart';

class Notify extends CuppsBase {
  final NotificationType notificationType;
  final String name;
  Notify({
    required int messageID,
    required String messageName,
    required this.notificationType,
    required this.name,
  }) : super(messageID: messageID, messageName: messageName);

  factory Notify.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    // String xmlns = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "xmlns").value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    // String xsi = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "xmlns:xsi").value;
    // String result = document.rootElement.childElements.first.attributes.firstWhere((p0) => p0.name.toString() == "result").value;

    NotificationType notifType = NotificationType.unknown;
    XmlElement notifyElement = document.findAllElements("notify").first;
    String name = "unknown";

    if (notifyElement.attributes
        .any((p0) => p0.name.toString() == "dataAvailable")) {
      if (notifyElement.attributes
              .firstWhere((p0) => p0.name.toString() == "dataAvailable")
              .value ==
          "true") {
        notifType = NotificationType.dataAvailable;
      }
    }
    if (notifyElement.attributes.isNotEmpty) {
      name = notifyElement.attributes.first.name.toString();
    } else {
      name = notifyElement.childElements.first.name.toString();
    }

    if (notifyElement.childElements.any(
        (element) => element.name.toString() == "deviceLockExpiredEvent")) {
      notifType = NotificationType.deviceLockExpire;
    }

    if (notifyElement.childElements.any(
        (element) => element.name.toString() == "deviceStatusNotification")) {
      notifType = NotificationType.deviceStatus;
    }

    return Notify(
        messageName: messageName,
        messageID: int.parse(messageID),
        notificationType: notifType,
        name: name);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName " />
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName : $xsi :  : $xmlns ";
  }
}

enum NotificationType {
  dataAvailable,
  deviceLockExpire,
  deviceStatus,
  unknown,
}
