import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import '../../classes/classes.dart';
import 'dart:io';

import 'package:xml/xml.dart';

import 'logger.dart';

class SocketService {
  SocketService();

  String logIdentifier = "Unkown";
  String logSuffix = "Unkown";
  int messageID = 0;
  Socket? _socket;
  void Function(CuppsBase, String)? requestHandler;
  void Function()? onConnectionClose;
  String temp = "";
  final Map<int, Completer<String>> _requests = {};

  bool get isConnected => (_socket != null);

  initialize(
      {required String ip,
      required int port,
      required void Function(CuppsBase, String) rh,
      required void Function() onClose,
      required String logID,
      required String logSuff}) async {
    if (_socket != null) {
      await _socket!.close();
    }
    messageID = 1;
    requestHandler = rh;
    onConnectionClose = onClose;
    logIdentifier = logID;
    logSuffix = logSuff;
    Socket s = await Socket.connect(ip, port);
    try {
      s.listen(_onSocketMessage, cancelOnError: true, onError: (e) {
        log("ERROR ON SOCKET $e");
        disconnect();
      });
      _socket = s;
    }catch (e){
      log("ERROR ON LISTENING TO SOCKET $e");
    }
    log("$logIdentifier Socket Service Connected");
  }

  Future<String> sendSocketMessage(String data) async {
    try {
      if (isConnected) {
        final completer = Completer<String>();
        final requestId = messageID;
        messageID++;
        _requests[requestId] = completer;

        XmlDocument formattedDOC = XmlDocument.parse(data);
        String formattedXML = formattedDOC.toXmlString(pretty: true);
        String header = generateHeaderForXML(formattedXML);
        String request = header + formattedXML;
        // print("Sending $messageID --->$data");
        await Logger.writeToLog(header + formattedXML, from: "Application", to: logIdentifier, logSuffix: logSuffix);

        _socket!.write(request);

        return completer.future;
      } else {
        return "Socket Closed";
      }
    } catch (e) {
      log("Sending to Socket ERROR: $e");
      rethrow;
    }
  }

  Future<void> respond(String data) async {
    try {
      if (isConnected) {
        String header = generateHeaderForXML(data);
        String request = header + data;
        _socket!.write(request);
      } else {
        log("Socket Closed");
      }
    } catch (e) {
      log("Sending to Socket ERROR: $e");
      rethrow;
    }
  }

  Future<void> _onSocketMessage(data) async {
    String xml = utf8.decode(data);
    String headerShouldBe = (temp + xml).split("<").first;
    bool messageENDED = xml.trimer.endsWith("</cupps>");
    String received = (temp + xml).replaceFirst(headerShouldBe, "");
    // print("received $received from Platform");

    try {
      if (messageENDED) {
        // print("MESSAGE COMPLETE");
        temp = "";
        String beautifulXMl = XmlDocument.parse(received).toXmlString(pretty: true);
        await Logger.writeToLog(beautifulXMl, to: "Application", from: logIdentifier, logSuffix: logSuffix);

        CuppsBase response = CuppsBase.fromXML(received);
        int messageID = response.messageID;
        if (_requests.containsKey(messageID)) {
          _requests[messageID]!.complete(received);
          _requests.remove(messageID);
        } else {
          requestHandler!(response, received);
        }
      } else {
        log("MESSAGE IS COMING");
        temp += xml;
      }
    } catch (e) {
      log("$e--->$received");
    }
  }

  static String generateHeaderForXML(String xml) {
    int length = utf8.encode(xml).length;
    String temp = length.toRadixString(16).padLeft(6, "0");
    String header = "0100" + temp;
    return header;
  }

  void disconnect() async {
    if (_socket != null) {
      // _instance.socket!.close();
      // await Future.delayed(const Duration(milliseconds: 100));
      await _socket!.close();
      log("Socket Service Closed");
      _socket?.destroy();
      log("Socket Service Destroyed");
      _socket = null;
      onConnectionClose!();
    }
  }
}
